﻿import com.robertpenner.effects.MotionCamera;
import mx.utils.Delegate;

/**
* The FractalTree class.
* 
* @author Robert Penner (AS 1.0)
* © 2002 Robert Penner
* http://www.robertpenner.com
* 
* @author Mark Walters (AS 2.0)
* © 2006 DigitalFlipbook
* http://www.digitalflipbook.com
* 
* @version	1.0
* 
* @since	ActionScript 2.0; Flash Player 6
*/
class com.robertpenner.profmx.fractalTree.FractalTree extends MovieClip {
	
	/*
	* ON STAGE
	*/
	private var trunk:MovieClip;
	private var time:TextField;
	private var maxTime:TextField;
	private var x_txt:TextField;
	private var y_txt:TextField;
	
	/*
	* CLASS PROPERTIES
	*/
	private var $posers:Array;
	private var $xCam:MotionCamera;
	private var $yCam:MotionCamera;
	
	/*
	* CLIP PARAMETERS
	*/
	private var $clipParameters:Object = {maxLevels: 1, maxBranches: 1, filmFrames: 1};
	
	/*
	* INSPECTABLE PROPERTIES
	*/
	private var $maxLevels:Number;
	private var $maxBranches:Number;
	private var $filmFrames:Number;
	
	
	/*
	* *********************************************************
	* CONSTRUCTOR
	* *********************************************************
	*
	*/
	
	function FractalTree () {
		this.initFromClipParameters();
	}
	
	
	/*
	* *********************************************************
	* PRIVATE METHODS
	* *********************************************************
	* 
	*/
	
	private function initFromClipParameters ():Void {
		var key:String, found:Boolean = false;
		for(key in $clipParameters) {
			if(this.hasOwnProperty(key)) {
				found = true;
				this["def_" + key] = this[key];
				delete this[key];
			}
		}
		if(found) {
			for(key in $clipParameters) {
				var v = this["def_" + key];
				if(v != undefined) {
					this[key] = v;
				}
			}
		}
	}
	
	private function init ():Void {
		this.$posers = [];
		for (var i=1; i <= this.maxBranches; i++) {
			this.$posers[i] = this["poser"+i];
			this.$posers[i]._visible = false;
		}
		
		this.$xCam = new MotionCamera (this.x_txt, "text", this.filmFrames);
		this.$xCam.setActor (_level0, "_xmouse");
		this.$xCam.looping = true;
		this.$xCam.onMotionChanged = Delegate.create(this, onMotionChanged);
		
		this.$yCam = new MotionCamera (this.y_txt, "text", this.filmFrames);
		this.$yCam.setActor (_level0, "_ymouse");
		this.$yCam.looping = true;
		
		this.maxTime.text = this.filmFrames.toString();
	}
	
	
	/*
	* *********************************************************
	* EVENT HANDLERS
	* *********************************************************
	* 
	*/
	
	private function onMotionChanged ():Void {
		this.$xCam.duration = Number(this.maxTime.text);
		this.$yCam.duration = Number(this.maxTime.text);
		this.time.text = this.$xCam.time.toString();
	}
	
	private function onMouseDown ():Void {
		this.$xCam.startRecord();
		this.$yCam.startRecord();
	}
	
	private function onMouseUp ():Void {
		this.$xCam.stopRecord();
		this.$yCam.stopRecord();
	}
	
	
	/*
	* *********************************************************
	* GETTERS/SETTERS
	* *********************************************************
	* 
	*/
	
	/**
	* The maxLevels property of the FractalTree.
	* 
	* @usage
	* <pre><code>
	* trace (fractalTree.maxLevels);
	* </code></pre>
	*/
	[Inspectable(type=Number, defaultValue=6)]
	public function get maxLevels ():Number {
		return this.$maxLevels;
	}
	public function set maxLevels (ml:Number):Void {
		 this.$maxLevels = ml;
	}
	
	/**
	* The maxBranches property of the FractalTree.
	* 
	* @usage
	* <pre><code>
	* trace (fractalTree.maxBranches);
	* </code></pre>
	*/
	[Inspectable(type=Number, defaultValue=2)]
	public function get maxBranches ():Number {
		return this.$maxBranches;
	}
	public function set maxBranches (mb:Number):Void {
		 this.$maxBranches = mb;
	}
	
	/**
	* The filmFrames property of the FractalTree.
	* 
	* @usage
	* <pre><code>
	* trace (fractalTree.filmFrames);
	* </code></pre>
	*/
	[Inspectable(type=Number, defaultValue=150)]
	public function get filmFrames ():Number {
		return this.$filmFrames;
	}
	public function set filmFrames (ff:Number):Void {
		 this.$filmFrames = ff;
	}
	
	/**
	* The posers property of the FractalTree.
	* 
	* @usage
	* <pre><code>
	* trace (fractalTree.posers);
	* </code></pre>
	*/
	public function get posers ():Array {
		return this.$posers;
	}
	
	/**
	* The xCam property of the FractalTree.
	* 
	* @usage
	* <pre><code>
	* trace (fractalTree.xCam);
	* </code></pre>
	*/
	public function get xCam ():MotionCamera {
		return this.$xCam;
	}
	
	/**
	* The yCam property of the FractalTree.
	* 
	* @usage
	* <pre><code>
	* trace (fractalTree.yCam);
	* </code></pre>
	*/
	public function get yCam ():MotionCamera {
		return this.$yCam;
	}
	
}
